@@ -18,7 +18,7 @@
     License along with this library; if not, write to the Free
     Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-    $Id: util.c,v 1.133 2005/02/23 09:13:06 tml Exp $
+    $Id: util.c 576 2007-05-15 06:48:50Z markmc $
 
 ***************************************************************************/
 #include <assert.h>
@@ -249,9 +249,7 @@ int IDL_parse_filename (const char *filename, const char *cpp_args,
 	}
 #endif
 
-	if (!filename ||
-	    !tree ||
-	    (tree == NULL && ns != NULL)) {
+	if (!filename || !tree) {
 		errno = EINVAL;
 		return -1;
 	}
@@ -376,8 +374,7 @@ int IDL_parse_filename (const char *filename, const char *cpp_args,
 	__IDL_msgcb = NULL;
 
 	if (rv != 0 || !__IDL_is_okay) {
-		if (tree)
-			*tree = NULL;
+		*tree = NULL;
 
 		if (ns)
 			*ns = NULL;
@@ -388,10 +385,7 @@ int IDL_parse_filename (const char *filename, const char *cpp_args,
 	if (__IDL_flags & IDLF_PREFIX_FILENAME)
 		IDL_ns_prefix (__IDL_root_ns, filename);
 
-	if (tree)
-		*tree = __IDL_root;
-	else
-		IDL_tree_free (__IDL_root);
+	*tree = __IDL_root;
 
 	if (ns)
 		*ns = __IDL_root_ns;
@@ -416,13 +410,11 @@ int IDL_parse_filename_with_input (const char *filename,
 	GSList *slist;
 	int rv;
 
-	if (!filename || !input_cb || !tree ||
-	    (tree == NULL && ns != NULL)) {
+	if (!filename || !input_cb || !tree) {
 		errno = EINVAL;
 		return -1;
 	}
 
-
 	IDL_parse_setup(parse_flags, max_msg_level);
 
 	__IDL_msgcb = msg_cb;
@@ -466,8 +458,7 @@ int IDL_parse_filename_with_input (const char *filename,
 	__IDL_msgcb = NULL;
 
 	if (rv != 0 || !__IDL_is_okay) {
-		if (tree)
-			*tree = NULL;
+		*tree = NULL;
 
 		if (ns)
 			*ns = NULL;
@@ -483,10 +474,7 @@ int IDL_parse_filename_with_input (const char *filename,
 	if (__IDL_flags & IDLF_PREFIX_FILENAME)
 		IDL_ns_prefix (__IDL_root_ns, filename);
 
-	if (tree)
-		*tree = __IDL_root;
-	else
-		IDL_tree_free (__IDL_root);
+	*tree = __IDL_root;
 
 	if (ns)
 		*ns = __IDL_root_ns;
@@ -499,13 +487,13 @@ int IDL_parse_filename_with_input (const char *filename,
 void yyerrorl (const char *s, int ofs)
 {
 	int line = __IDL_cur_line - 1 + ofs;
-	gchar *filename = NULL;
+	gchar *freeme = NULL, *filename = NULL;
 
 	if (__IDL_cur_filename) {
 #ifdef HAVE_CPP_PIPE_STDIN
 		filename = __IDL_cur_filename;
 #else
-		filename = g_basename (__IDL_cur_filename);
+		freeme = filename = g_path_get_basename (__IDL_cur_filename);
 #endif
 	} else
 		line = -1;
@@ -514,8 +502,10 @@ void yyerrorl (const char *s, int ofs)
 	__IDL_is_okay = FALSE;
 
 	/* Errors are counted, even if not printed */
-	if (__IDL_max_msg_level < IDL_ERROR)
+	if (__IDL_max_msg_level < IDL_ERROR) {
+		g_free (freeme);
 		return;
+	}
 
 	if (__IDL_msgcb)
 		(*__IDL_msgcb)(IDL_ERROR, __IDL_nerrors, line, filename, s);
@@ -525,12 +515,13 @@ void yyerrorl (const char *s, int ofs)
 		else
 			fprintf (stderr, "Error: %s\n", s);
 	}
+	g_free (freeme);
 }
 
 void yywarningl (int level, const char *s, int ofs)
 {
 	int line = __IDL_cur_line - 1 + ofs;
-	gchar *filename = NULL;
+	gchar *freeme = NULL, *filename = NULL;
 
 	/* Unprinted warnings are not counted */
 	if (__IDL_max_msg_level < level)
@@ -540,7 +531,7 @@ void yywarningl (int level, const char *s, int ofs)
 #ifdef HAVE_CPP_PIPE_STDIN
 		filename = __IDL_cur_filename;
 #else
-		filename = g_basename (__IDL_cur_filename);
+		freeme = filename = g_path_get_basename (__IDL_cur_filename);
 #endif
 	} else
 		line = -1;
@@ -555,6 +546,7 @@ void yywarningl (int level, const char *s, int ofs)
 		else
 			fprintf (stderr, "Warning: %s\n", s);
 	}
+	g_free (freeme);
 }
 
 void yyerror (const char *s)
@@ -1876,10 +1868,18 @@ static void IDL_tree_free_real (IDL_tree p)
 		g_free (IDL_STRING (p).value);
 		break;
 
+	case IDLN_WIDE_STRING:
+		g_free (IDL_WIDE_STRING (p).value);
+		break;
+
 	case IDLN_CHAR:
 		g_free (IDL_CHAR (p).value);
 		break;
 
+	case IDLN_WIDE_CHAR:
+		g_free (IDL_WIDE_CHAR (p).value);
+		break;
+
 	case IDLN_IDENT:
 		g_free (IDL_IDENT (p).str);
 		g_free (IDL_IDENT_REPO_ID (p));
@@ -1944,9 +1944,12 @@ void IDL_tree_free (IDL_tree p)
 	case IDLN_TYPE_TYPECODE:
 	case IDLN_FIXED:
 	case IDLN_STRING:
+	case IDLN_WIDE_STRING:
 	case IDLN_CHAR:
+	case IDLN_WIDE_CHAR:
 	case IDLN_IDENT:
 	case IDLN_CODEFRAG:
+	case IDLN_SRCFILE:
 		__IDL_tree_free (p);
 		break;
 
@@ -3350,7 +3353,7 @@ struct IDL_emit_node {
 	IDL_tree_func post;
 };
 
-static const struct IDL_emit_node * const IDL_get_IDL_emission_table (void)
+static struct IDL_emit_node * IDL_get_IDL_emission_table (void)
 {
 	static gboolean initialized = FALSE;
 	static struct IDL_emit_node table[IDLN_LAST];
